package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.BaseController;
import com.yunzhi.shigongli.common.Constants;
import com.yunzhi.shigongli.common.ResponseBean;
import com.yunzhi.shigongli.common.StringUtils;
import com.yunzhi.shigongli.entity.TaHotWord;
import com.yunzhi.shigongli.service.ITaHotWordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
    * 热词 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "热词")
@RestController
@RequestMapping("/")
public class TaHotWordController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaHotWordController.class);

    @Autowired
    public ITaHotWordService iTaHotWordService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/hot-word",method= RequestMethod.GET)
    @ApiOperation(value="管理端热词列表", notes = "管理端热词列表", httpMethod = "GET", response = TaHotWord.class)
    public ResponseBean tdHotWordList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{

        IPage<TaHotWord> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<TaHotWord> queryWrapper = new QueryWrapper<>();
        queryWrapper.gt("status", Constants.STATUS_DELETED);
        queryWrapper.orderByAsc("search_num");
        queryWrapper.orderByAsc("weight");

        IPage<TaHotWord> result = iTaHotWordService.page(pg, queryWrapper);
        return ResponseBean.success(result);
    }

    @GetMapping("/wx/hot-word")
    @ApiOperation(value="小程序端热词列表", notes = "小程序端热词列表", httpMethod = "GET", response = TaHotWord.class)
    public ResponseBean getWxList() throws Exception{

        QueryWrapper<TaHotWord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", Constants.STATUS_NORMAL);
        queryWrapper.orderByAsc("weight");
        queryWrapper.orderByAsc("create_date");

        List<TaHotWord> result = iTaHotWordService.list(queryWrapper);
        return ResponseBean.success(result);
    }


    @PostMapping("/admin/hot-word")
    @ApiOperation(value="新增热词", notes = "新增热词", httpMethod = "POST", response = TaHotWord.class)
    public ResponseBean saveHotWord(@ApiParam("热词对象") @RequestBody TaHotWord taHotWord) throws Exception{
        if (null == taHotWord || StringUtils.isEmpty(taHotWord.getWord())) {
            return ResponseBean.error("热搜词不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        QueryWrapper<TaHotWord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("word", taHotWord.getWord());
        queryWrapper.gt("status", Constants.STATUS_DELETED);
        List<TaHotWord> lst = iTaHotWordService.list(queryWrapper);

        if (null != lst && lst.size() > 0) {
            return ResponseBean.error("热搜词已存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (null == taHotWord.getWeight()) {
            taHotWord.setWeight(0);
        }
        taHotWord.setSearchNum(0);
        taHotWord.setStatus(Constants.STATUS_NORMAL);
        taHotWord.setCreateDate(LocalDateTime.now());

        iTaHotWordService.save(taHotWord);
        return ResponseBean.success(taHotWord);
    }

    @PutMapping("/admin/hot-word/{id}")
    @ApiOperation(value="更新热词", notes = "更新热词", httpMethod = "PUT", response = TaHotWord.class)
    public ResponseBean updateHotWord(@ApiParam("热词ID") @PathVariable String id,
                                    @ApiParam("热词对象") @RequestBody TaHotWord taHotWord) throws Exception{
        TaHotWord origin = iTaHotWordService.getById(id);
        if (null == origin || Constants.STATUS_DELETED.equals(origin.getStatus())) {
            return ResponseBean.error("未找到对应热词", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (null == taHotWord || StringUtils.isEmpty(taHotWord.getWord())) {
            return ResponseBean.error("热搜词不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 理论上应该只允许修改权重
        origin.setWeight(taHotWord.getWeight());
        if (null == origin.getWeight()) {
            origin.setWeight(0);
        }

        iTaHotWordService.updateById(origin);
        return ResponseBean.success(origin);
    }

    @PutMapping("/admin/hot-word/{id}/on")
    @ApiOperation(value="发布热词", notes = "发布热词", httpMethod = "PUT", response = TaHotWord.class)
    public ResponseBean publishHotWord(@ApiParam("热词ID") @PathVariable String id) throws Exception{
        TaHotWord taHotWord = iTaHotWordService.getById(id);
        if (null == taHotWord || Constants.STATUS_DELETED.equals(taHotWord.getStatus())) {
            return ResponseBean.error("未找到对应热词", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
        taHotWord.setStatus(Constants.STATUS_NORMAL);

        iTaHotWordService.updateById(taHotWord);
        return ResponseBean.success(taHotWord);
    }

    @PutMapping("/admin/hot-word/{id}/off")
    @ApiOperation(value="关闭热词", notes = "关闭热词", httpMethod = "PUT", response = TaHotWord.class)
    public ResponseBean unPublishHotWord(@ApiParam("热词ID") @PathVariable String id) throws Exception{
        TaHotWord taHotWord = iTaHotWordService.getById(id);
        if (null == taHotWord || Constants.STATUS_DELETED.equals(taHotWord.getStatus())) {
            return ResponseBean.error("未找到对应热词", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
        taHotWord.setStatus(Constants.STATUS_READY);

        iTaHotWordService.updateById(taHotWord);
        return ResponseBean.success(taHotWord);
    }

    @DeleteMapping("/admin/hot-word/{id}")
    @ApiOperation(value="删除热词", notes = "删除热词", httpMethod = "DELETE", response = TaHotWord.class)
    public ResponseBean deleteHotWord(@ApiParam("热词ID") @PathVariable String id) throws Exception{
        iTaHotWordService.removeLogicById(id);
        return ResponseBean.success(null);
    }
}
