package com.yunzhi.shigongli.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.yunzhi.shigongli.common.*;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.mapper.*;
import com.yunzhi.shigongli.service.ITaOrderPayService;
import com.yunzhi.shigongli.service.ITaOrderService;

import com.yunzhi.shigongli.service.ITaVerifyTargetService;
import com.yunzhi.shigongli.vo.OrderItem;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */
@Slf4j
@Service
public class TaOrderServiceImpl extends BaseServiceImpl<TaOrderMapper, TaOrder> implements ITaOrderService {

    @Autowired
    TaShopPackageMapper taShopPackageMapper;

    @Autowired
    TaShopPackageSettingMapper packageSettingMapper;

    @Autowired
    TaOrderMapper taOrderMapper;

    @Autowired
    TaOrderSubMapper taOrderSubMapper;

    @Autowired
    TaOrderCommissionMapper commissionMapper;

    @Autowired
    TaOrderPayMapper orderPayMapper;

    @Autowired
    TaPersonRecommenderMapper personRecommenderMapper;

    @Autowired
    public ITaOrderPayService iTaOrderPayService;

    @Autowired
    public ITaVerifyTargetService iTaVerifyTargetService;

    /**
     * 下单支付
     * @param person
     * @param orderItems
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public TaOrder order(TaPerson person, OrderItem[] orderItems) throws Exception {
        LocalDateTime now = LocalDateTime.now();

        // 是否首单
        TaOrder lastOrder = getPersonLastOrder(person.getPersonId());
        boolean isFirstOrder = null == lastOrder;

        // 先生成主订单
        TaOrder taOrder = new TaOrder();
        taOrder.setOrderId(getOrderNo());
        taOrder.setDescription("套餐");
        taOrder.setPersonId(person.getPersonId());
        // 默认未支付
        taOrder.setStatus(Constants.ORDER_READY);
        taOrder.setCreateDate(now);

        // 子订单列表
        Integer charge = createAndSaveOrderSubList(person, taOrder, orderItems, isFirstOrder);
        taOrder.setCharges(charge);
        save(taOrder);

        return taOrder;
    }

    /**
     * 取消订单
     * @param taOrder
     * @param taPerson
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void cancelOrder(TaOrder taOrder, TaPerson taPerson) throws Exception {
        List<TaOrderSub> orderSubList = taOrderSubMapper.getListByOrder(taOrder.getOrderId());
        if (null == orderSubList || orderSubList.size() < 1) {
            throw new Exception("未找到订单详情");
        }

        // 如果有已核销的, 则不能取消
        Object[] o1 = orderSubList.stream().filter(x -> x.getIsVerified()).toArray();
        if (o1.length > 0) {
            throw new Exception("订单存在已核销内容, 不能取消");
        }

        // 如果已支付, 则不能取消
        if (Constants.ORDER_PAYED.equals(taOrder.getStatus())) {
            throw new Exception("订单已支付, 请先申请退款");
        }

        // 只有未支付的, 或者已退款的可以取消
        if (!Constants.ORDER_READY.equals(taOrder.getStatus())
            && !Constants.ORDER_REFUND.equals(taOrder.getStatus())) {
            log.error("退款订单【{}】, 状态为 {}", taOrder.getOrderId(), taOrder.getStatus());
            throw new Exception("订单状态异常, 请联系相关人员");
        }

        // 只需要更新状态即可
        taOrderMapper.updateStatus(taOrder.getOrderId(), Constants.ORDER_CANCEL);
        taOrderSubMapper.updateStatusByOrder(taOrder.getOrderId(), Constants.ORDER_CANCEL);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void refundOrder(TaOrder taOrder, TaPerson taPerson, Integer totalCashback) throws Exception {

        taOrder.setStatus(Constants.ORDER_REFUND);
        taOrderMapper.updateById(taOrder);

        // 只需要更新状态即可
//        taOrderMapper.updateStatus(taOrder.getOrderId(), Constants.ORDER_REFUND);
        taOrderSubMapper.updateStatusByOrder(taOrder.getOrderId(), Constants.ORDER_REFUND);

        iTaOrderPayService.refundOrder(taPerson, taOrder, totalCashback);
        // 删除所有待核销内容
        iTaVerifyTargetService.removeBy(taOrder.getOrderId(), null);
    }

    /**
     * 获取订单号
     * @return
     */
    private String getOrderNo() {
        LocalDateTime now = LocalDateTime.now();
        String today = DateUtils.toString(now, "yyyyMMddHHmm");
        String orderNo = null;
        do {
            String random = StringUtils.random(4);
            orderNo = String.format("%s%s", today, random);
            TaOrder taOrder = getById(orderNo);
            if (null == taOrder) {
                break;
            }
        } while (true);

        return orderNo;
    }

    /**
     * 生成并保存子订单
     * @param taOrder
     * @param orderItems
     * @return
     * @throws Exception
     */
    private Integer createAndSaveOrderSubList(TaPerson taPerson, TaOrder taOrder, OrderItem[] orderItems, boolean isFirstOrder) throws Exception {
        LocalDateTime now = LocalDateTime.now();

        Integer totalCharge = 0;
        for (int i = 0; i < orderItems.length; i += 1) {
            OrderItem item = orderItems[i];
            String subOrderId = taOrder.getOrderId() + "-" + (i + 1);
            TaOrderSub orderSub = new TaOrderSub();
            orderSub.setSubOrderId(subOrderId);
            orderSub.setOrderId(taOrder.getOrderId());
            // 默认未核销
            orderSub.setIsVerified(false);
            // 未支付
            orderSub.setStatus(Constants.ORDER_READY);
            // 时间
            orderSub.setCreateDate(now);

            if (StringUtils.isEmpty(item.getItemId())) {
                throw new Exception("订单中存在空的订单项");
            }
            if (null == item.getPrice() || item.getPrice() < 0) {
                throw new Exception("订单中存在不正常的金额");
            }
            if (null == item.getAmount() || item.getAmount() < 0) {
                throw new Exception("订单中存在不正常的数量");
            }

            // 套餐ID
            String packageId = item.getItemId();
            if (StringUtils.isEmpty(packageId)) {
                throw new Exception("订单中存在空的订单项");
            }

            // 套餐
            TaShopPackage shopPackage = taShopPackageMapper.selectById(packageId);
            if (null == shopPackage || !Constants.STATUS_NORMAL.equals(shopPackage.getStatus())) {
                throw new Exception("订单中存在异常状态套餐");
            }
            if (shopPackage.getCashback() >= shopPackage.getActualPrice()) {
                throw new Exception("订单中存在异常套餐, 请联系管理员");
            }

            // 当前套餐的店铺是否当前下单人的推荐人
            boolean isRecommend = Constants.RECOMMEND_SHOP.equals(taPerson.getFromType())
                    && shopPackage.getShopId().equals(taPerson.getRecommender());

            // 当前人员是否有有效的推荐人
            boolean hasRecommend = checkPersonRecommender(taPerson);

            orderSub.setShopId(shopPackage.getShopId());
            orderSub.setPackageId(packageId);
            String desc = StringUtils.ifNull(shopPackage.getDescription(), "套餐项目");
            orderSub.setDescription(desc.length() > 128 ? desc.substring(0, 128) : desc);

            // 价格
            if (!item.getPrice().equals(shopPackage.getActualPrice())) {
                throw new Exception("订单中存在变化的价格, 请刷新重试");
            }

            orderSub.setUnitPrice(item.getPrice());
            orderSub.setCharges(item.getPrice() * item.getAmount());
            totalCharge += orderSub.getCharges();

            // 数量
            orderSub.setAmount(item.getAmount());

            // 返现设置
            orderSub.setCashback(shopPackage.getCashback());

            // 新客户首次推荐店铺下单, 商铺不需要给平台分成
            boolean needCommission = !isFirstOrder || !isRecommend;

            // 套餐分成设置
            TaShopPackageSetting packageSetting = packageSettingMapper.selectById(packageId);
            if (null != packageSetting && needCommission) {
                // 参与分成金额 = 总金额 - 返现金额
                Integer charges = orderSub.getCharges() - orderSub.getCashback();

                // 引流分成
                Integer trafficRate = hasRecommend ? packageSetting.getTrafficRate() : 0;
                Integer trafficCommission = (int) (charges * (trafficRate.floatValue() / 1000));
                // 平台分成
                Integer platformRate = packageSetting.getPlatformRate();
                Integer platformCommission = (int) (charges * (platformRate.floatValue() / 1000));
                // 总提成
                Integer commissionCharge = trafficCommission + platformCommission;
                if (commissionCharge < 0 || commissionCharge >= charges) {
                    log.error("下单失败; 套餐[{}]抽成与返现设置有问题", packageId);
                    throw new Exception("订单中存在异常套餐, 请联系相关人员");
                }

                if (trafficCommission > 0) {
                    createAndSaveOrderCommission(orderSub, trafficCommission, Constants.COMMISSION_TRAFFIC);
                }

                if (platformCommission > 0) {
                    createAndSaveOrderCommission(orderSub, trafficCommission, Constants.COMMISSION_PLATFORM);
                }

                orderSub.setCommissionCharge(commissionCharge);
            } else {
                orderSub.setCommissionCharge(0);
            }

            taOrderSubMapper.insert(orderSub);
        }

        return totalCharge;
    }

    private boolean checkPersonRecommender(TaPerson taPerson) {
        TaPersonRecommender acitveRecommender = personRecommenderMapper.getAcitveRecommender(taPerson.getPersonId());
        return null != acitveRecommender;
    }

    /**
     * 获取人员最近的一笔订单
     * @param personId
     * @return
     */
    private TaOrder getPersonLastOrder(String personId) {
        LambdaQueryWrapper<TaOrder> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaOrder::getPersonId, personId);
        queryWrapper.gt(TaOrder::getStatus, Constants.STATUS_DELETED);
        queryWrapper.orderByDesc(TaOrder::getCreateDate);
        queryWrapper.last("limit 1");

        return getOne(queryWrapper);
    }

    /**
     * 生成并保存抽成表
     * @param orderSub
     * @param charge
     * @param commissionType
     * @throws Exception
     */
    private void createAndSaveOrderCommission(TaOrderSub orderSub, Integer charge, String commissionType) throws Exception {
        LocalDateTime now = LocalDateTime.now();
        TaOrderCommission orderCommission = new TaOrderCommission();
        orderCommission.setOrderId(orderSub.getOrderId());
        orderCommission.setSubOrderId(orderSub.getSubOrderId());
        orderCommission.setCommissionType(commissionType);
        orderCommission.setCharge(charge);
        orderCommission.setCreateDate(now);
        commissionMapper.insert(orderCommission);
    }
}
